/*
 *************************************************************************
 *                                                                       *
 * Copyright 2000 Compaq Computer Corporation                            *
 *                                                                       *
 * COMPAQ Registered in U.S. Patent and Trademark Office.                *
 *                                                                       *
 *************************************************************************
 * IMPORTANT: Carefully read the License Terms below before              *
 * proceeding.  By use of these materials you agree to these terms.      *
 * If you do not agree to these terms, you may not use this software or  *
 * the accompanying documentation.                                       *
 *************************************************************************
 * LICENSE TERMS                                                         *
 * 1. GRANT                                                              *
 * Compaq Computer Corporation ("COMPAQ") grants you the right to use,   *
 * modify, and distribute the following source code (the "Software")     *
 * on any number of computers. You may use the Software as part of       *
 * creating a software program or product intended for commercial or     *
 * non-commercial distribution in machine-readable source code, binary,  *
 * or executable formats. You may distribute the Software as             *
 * machine-readable source code provided this license is not removed     *
 * from the Software and any modifications are conspicuously indicated.  *
 * 2. COPYRIGHT                                                          *
 * The Software is owned by COMPAQ and its suppliers and is protected by *
 * copyright laws and international treaties.  Your use of the Software  *
 * and associated documentation is subject to the applicable copyright   *
 * laws and the express rights and restrictions of these terms.          *
 * 3. RESTRICTIONS                                                       *
 * You may not remove any copyright, trademark, or other proprietary     *
 * notices from the Software or the associated  documentation.           *
 * You are responsible for compliance with all applicable export or      *
 * re-export control laws and regulations if you export the Software.    *
 * This license is governed by and is to be construed under the laws     *
 * of the State of Texas.                                                *
 *                                                                       *
 * DISCLAIMER OF WARRANTY AND LIABILITY                                  *
 * Compaq shall not be liable for technical or editorial errors or       *
 * omissions contained herein. The information contained herein is       *
 * subject to change without notice.                                     *
 *                                                                       *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.       *
 * THE ENTIRE RISK ARISING OUT OF THE USE OF THIS SOFTWARE REMAINS WITH  *
 * RECIPIENT.  IN NO EVENT SHALL COMPAQ BE LIABLE FOR ANY DIRECT,        *
 * CONSEQUENTIAL, INCIDENTAL, SPECIAL, PUNITIVE OR OTHER DAMAGES         *
 * WHATSOEVER (INCLUDING WITHOUT LIMITATION DAMAGES FOR LOSS OF BUSINESS *
 * PROFITS, BUSINESS INTERRUPTION, OR LOSS OF BUSINESS INFORMATION),     *
 * EVEN IF COMPAQ HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES    *
 * AND WHETHER IN AN ACTION OF CONTRACT OR TORT INCLUDING NEGLIGENCE.    *
 *                                                                       *
 * If you have any questions concerning this license, please contact:    *
 * Compaq Computer Corporation, Software Business Practices, ZKO1-2/D22, *
 * 110 Spit Brook Road, Nashua, NH. 03062-2698.                          *
 *                                                                       *
 *************************************************************************
 */

#if defined(__ALPHA) && __CRTL_VER >= 70301000
#include "vms_jackets.h"
#include <limits.h>
#include <stdio.h>
#include <stat.h>
#include <time.h>
#include <file.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <unixio.h>

/*
** There are actually two stat jackets in here. The first is the normal
** file parsing one, and the second is one that handles "/" and "/DDcu".
** The first calls the second so both are in effect.
*/

/*
** Debugging/tracing macros.
*/
#ifdef DEBUG
const char debug_lstat[]=VMS_PREFIX "DEBUG_LSTAT";
#define DPRINT(s) if (getenv(debug_lstat)) printf(s)
#define DPRINT_1(s,p1) if (getenv(debug_lstat)) printf(s,p1)
#define DPRINT_2(s,p1,p2) if (getenv(debug_lstat)) printf(s,p1,p2)
#define DPRINT_3(s,p1,p2,p3) if (getenv(debug_lstat)) printf(s,p1,p2,p3)
#define DPRINT_4(s,p1,p2,p3,p4) if (getenv(debug_lstat)) printf(s,p1,p2,p3,p4)
#else
#define DPRINT(s)
#define DPRINT_1(s,p1)
#define DPRINT_2(s,p1,p2)
#define DPRINT_3(s,p1,p2,p3)
#define DPRINT_4(s,p1,p2,p3,p4)
#endif

/*
** This is the template for the root lstat structure. Of course, such a
** thing doesn't exist on OpenVMS, so this is pure fabrication.
**
** Notice that because we need to be able to return a stat struct for our
** fabricated root, we need to tie its fileid to some unique device name.
** We could use anything, and it probably doesn't need to even be a valid
** device, but we use the NULL device. Hey, its about time someone used
** the poor null device.
*/
char lstat_root_dev[] = "_NLA0";
struct stat lstat_root =
{
  NULL,                 /* st_dev - pointer to physical device name */
  4, 4, 0,              /* st_ino[3] - 3 words to receive fid */
  S_IFDIR |             /* st_mode - file type is directory*/
  S_IRWXU |             /* mode is   SYSTEM:RWED */
  S_IRGRP | S_IXGRP |   /*           GROUP:RE */
  S_IXOTH,              /*           WORLD:E */
  0,                    /* st_nlink - for compatibility - not really used */
  0x10001,              /* st_uid - from ACP - QIO uic field */
  0x1,                  /* st_gid - group number extracted from st_uid */
  0,                    /* st_rdev - for compatibility - always zero */
  0,                    /* st_size - file size in bytes */
  0,                    /* st_atime - file access time; always same as st_mtime
  0,                    /* st_mtime - last modification time */
  0,                    /* st_ctime - file creation time */
  0,                    /* st_fab_rfm - record format - UDF */
  0,                    /* st_fab_rat - record attributes - none */
  0,                    /* st_fab_fsz - fixed header size */
  0                     /* st_fab_mrs - record size */
};

int GENERIC_LSTAT_JACKET2(const char *path, struct stat *buf) {

    int rv, len;
    char my_path[PATH_MAX+8];

    /*
    ** Check that we have the room to do our special stuff.
    */
    len=strlen(path);
    if (len <= PATH_MAX) {

        /*
        ** First off, tidy the spec up a little.
        */
        generic_tidy_spec(my_path, path);
        DPRINT_2("GENERIC_lstat_Jacket: %s, tidied %s\n",path,my_path);

        /*
        ** If its the root directory, then we return the fake lstat block. We
        ** update the access and modified times so the caller doesn't maintain
        ** any potentially out-of-date information.
        */
        if ( (strlen(my_path) == 1) && (my_path[0] == '/') ) {
            memmove(buf, &lstat_root, sizeof(lstat_root));
            buf->st_dev = lstat_root_dev;
            buf->st_atime = buf->st_mtime = time(NULL);
            DPRINT("GENERIC_lstat_Jacket: Was root\n");
            DPRINT_4("GENERIC_lstat_Jacket: (a) %s %d,%d,%d\n",buf->st_dev,
                buf->st_ino[0], buf->st_ino[1], buf->st_ino[2] );
            return 0;
        }

        /*
        ** If its an absolute spec with only one term, then try appending
        ** 000000 and see if that works. We have to do this first otherwise
        ** VMS's concept of the default directory can get in the way if
        ** we're dealing with a device name and that device is where the
        ** current default directory is. Eg. if the current default
        ** directory is DKA0:[X.Y] then a lstat of "/dka0" will actually
        ** return informatino about "/dka0/x/y".
        */
        if (generic_absone(my_path)) {
            char copy[PATH_MAX+7+1];
            strcpy(copy,my_path);
            len = strlen(copy);
            if (copy[len-1] == '/')
                copy[len-1] = '\0';
            strcat(copy,"/000000");
            rv=lstat(copy,buf);
            DPRINT_2("GENERIC_lstat_Jacket: absone true, lstat(%s)=%d\n",copy,rv);
            if (rv == 0) {
                DPRINT_4("GENERIC_lstat_Jacket: (b) %s %d,%d,%d\n",buf->st_dev,
                    buf->st_ino[0], buf->st_ino[1], buf->st_ino[2] );
                return 0;
            }
        }

        /*
        ** If its an absolute spec with only two terms, then try inserting
        ** 000000 and see if that works. We have to do this first otherwise
        ** VMS's concept of the default directory can get in the way if
        ** we're dealing with a something like /dka0/foo.dat.
        */
        if (generic_abstwo(my_path)) {
            char copy[PATH_MAX+7+1], *p;
            strcpy(copy,my_path);
	    p=strchr(&copy[1],'/');
	    *p='\0';
            strcat(copy,"/000000");
	    p=strchr(&my_path[1],'/');
            strcat(copy,p);
            rv=lstat(copy,buf);
            DPRINT_2("GENERIC_lstat_Jacket: abstwo true, lstat(%s)=%d\n",copy,rv);
            if (rv == 0) {
                DPRINT_4("GENERIC_lstat_Jacket: (b) %s %d,%d,%d\n",buf->st_dev,
                    buf->st_ino[0], buf->st_ino[1], buf->st_ino[2] );
                return 0;
            }
        }


    }

    /*
    ** Nothing special, so do a regular lstat, but on the tidied spec.
    */
    DPRINT_1("GENERIC_lstat_Jacket: calling normal lstat with %s\n",my_path);
    rv=lstat(my_path,buf);
    if (rv == 0) {
        DPRINT_4("GENERIC_lstat_Jacket: (c) %s %d,%d,%d\n",buf->st_dev,
            buf->st_ino[0], buf->st_ino[1], buf->st_ino[2] );
        return 0;
    }
    else {
        DPRINT_1("GENERIC_lstat_Jacket: (c) rv was %d\n",rv);
        return rv;
    }
}




/**********************************************************************/
/*  Jacket routines to handle VMS and decc limits                     */
/*                                                                    */
/*  Routine: GENERIC_LSTAT_JACKET                                      */
/*                                                                    */
/*  Jacket for: lstat                                                  */
/*                                                                    */
/**********************************************************************/
int GENERIC_LSTAT_JACKET(const char *file_spec,struct stat *__buf)
{
   char temp_string[TEMP_STRING_SIZE];
   char temp_dir_string[TEMP_STRING_SIZE];
   char *new_filespec;
   int temp_status,status;

   new_filespec = GENERIC_VMS_Filename_parser((char *)file_spec,
                                              temp_string,
					      GENERIC_K_STRING_TYPE_UNKNOWN,
					      &temp_status);
   GENERIC_TRACE("lstat_Jacket: Mapped:", file_spec, "To:", new_filespec);

   status = GENERIC_LSTAT_JACKET2(new_filespec,__buf);
   if (!(status==0)) /* could be a directory with several .'s check it out*/
	{
	   new_filespec = GENERIC_VMS_Filename_parser((char *)file_spec,
                                              temp_dir_string,
					      GENERIC_K_STRING_TYPE_DIRECTORY,
					      &temp_status);
	   if (strcmp(temp_string,temp_dir_string))/* diff lstat it now*/
		status = lstat(new_filespec,__buf);
	
	}
   return(status);
}

/*
** This is messy, but what can we do.
**
** stat.h prototypes a routine named lstat but does so with a __UTC prefix
** pragma. Since we have lstat defined to be GENERIC_LSTAT_JACKET the
** compiler/linker between themselves end up converting a lstat call to 
** a call to __UTC_GENERIC_LSTAT_JACKET. The simplest way to fix this
** is to give it such a routine, that does nothing except call our jacket.
*/
int __UTC_GENERIC_LSTAT_JACKET(const char *file_spec,struct stat *__buf) {
    return GENERIC_LSTAT_JACKET(file_spec,__buf);
}

#else
#include "vms_jackets.h"
#include <stat.h>
#include <stdio.h>

/*
** Usually stat can be used instead of lstat, but not always. So this routine
** calls stat so that it does something useful, but also prints out a warning
** message. When you see one of these messages, you should track down where
** its coming from and determine if changing it to a regular stat is the right
** thing to do or not.
*/

int GENERIC_LSTAT(const char *path, struct stat *buf)
{
#ifdef DEBUG
   fprintf(stderr,"ALERT - lstat stub called: %s\n",path);
#endif
   return(stat(path, buf));
}
#endif
